@@ -484,6 +484,11 @@ keywords["development"] = set(
     )
 )
 
+# For now we're defining 1.1 as 'development minus Directory' (the latter enforced in _parser).
+# We'll need to fork them when the development grammar diverges further.
+versions["1.1"] = versions["development"]
+keywords["1.1"] = keywords["development"]
+
 assert set(versions.keys()) == set(keywords.keys())
 
 
