@@ -12,9 +12,8 @@ import glob
 import threading
 import shutil
 import re
-from typing import Tuple, List, Dict, Optional, Callable, Iterable, Set, Any, Union
+from typing import Tuple, List, Dict, Optional, Callable, Set, Any, Union
 from contextlib import ExitStack
-from docker.errors import BuildError as DockerBuildError
 
 from .. import Error, Type, Env, Value, StdLib, Tree, Expr, _util
 from .._util import (
@@ -32,9 +31,8 @@ from .._util import (
 from .._util import StructuredLogMessage as _
 from . import config, _statusbar
 from .download import able as downloadable, run_cached as download
-from .cache import CallCache
-from .error import OutputError, Interrupted, Terminated, CommandFailed, RunFailed, error_json
-from .task_container import TaskContainer, new as new_task_container
+from .cache import CallCache, new as new_call_cache
+from .error import OutputError, Interrupted, Terminated, RunFailed, error_json
 
 
 def run_local_task(
@@ -59,6 +57,7 @@ def run_local_task(
                     (defaults to current working directory).
                     If the final path component is ".", then operate in run_dir directly.
     """
+    from .task_container import new as new_task_container  # delay heavy import
 
     _run_id_stack = _run_id_stack or []
     run_id = run_id or task.name
@@ -95,8 +94,9 @@ def run_local_task(
         write_values_json(inputs, os.path.join(run_dir, "inputs.json"))
 
         if not _run_id_stack:
-            cache = _cache or cleanup.enter_context(CallCache(cfg, logger))
+            cache = _cache or cleanup.enter_context(new_call_cache(cfg, logger))
             cache.flock(logfile, exclusive=True)  # no containing workflow; flock task.log
+            cache.flock(run_dir, exclusive=True)
         else:
             cache = _cache
         assert cache
@@ -104,11 +104,7 @@ def run_local_task(
         container = None
         try:
             cache_key = f"{task.name}/{task.digest}/{Value.digest_env(inputs)}"
-            cached = cache.get(
-                key=cache_key,
-                output_types=task.effective_outputs,
-                inputs=inputs,
-            )
+            cached = cache.get(cache_key, inputs, task.effective_outputs)
             if cached is not None:
                 for decl in task.outputs:
                     v = cached[decl.name]
@@ -322,7 +318,7 @@ def _eval_task_inputs(
     logger: logging.Logger,
     task: Tree.Task,
     posix_inputs: Env.Bindings[Value.Base],
-    container: TaskContainer,
+    container: "runtime.task_container.TaskContainer",
 ) -> Env.Bindings[Value.Base]:
 
     # Map all the provided input File & Directory paths to in-container paths
@@ -409,7 +405,7 @@ def _eval_task_runtime(
     cfg: config.Loader,
     logger: logging.Logger,
     task: Tree.Task,
-    container: TaskContainer,
+    container: "runtime.task_container.TaskContainer",
     env: Env.Bindings[Value.Base],
     stdlib: StdLib.Base,
 ) -> Dict[str, Union[int, str]]:
@@ -423,6 +419,8 @@ def _eval_task_runtime(
             raise Error.InputError(f"invalid default runtime setting {key} = {v}")
     for key, expr in task.runtime.items():
         runtime_values[key] = expr.eval(env, stdlib)
+    if "container" in runtime_values:  # alias
+        runtime_values["docker"] = runtime_values["container"]
     logger.debug(_("runtime values", **dict((key, str(v)) for key, v in runtime_values.items())))
     ans = {}
 
@@ -434,7 +432,11 @@ def _eval_task_runtime(
         dockerfile = "\n".join(elt.coerce(Type.String()).value for elt in dockerfile.value)
         ans["inlineDockerfile"] = dockerfile
     elif "docker" in runtime_values:
-        ans["docker"] = runtime_values["docker"].coerce(Type.String()).value
+        docker_value = runtime_values["docker"]
+        if isinstance(docker_value, Value.Array) and len(docker_value.value):
+            # TODO: ask TaskContainer to choose preferred candidate
+            docker_value = docker_value.value[0]
+        ans["docker"] = docker_value.coerce(Type.String()).value
 
     host_limits = container.__class__.detect_resource_limits(cfg, logger)
     if "cpu" in runtime_values:
@@ -487,7 +489,9 @@ def _eval_task_runtime(
     if ans:
         logger.info(_("effective runtime", **ans))
     unused_keys = list(
-        key for key in runtime_values if key not in ("cpu", "memory") and key not in ans
+        key
+        for key in runtime_values
+        if key not in ("cpu", "memory", "docker", "container") and key not in ans
     )
     if unused_keys:
         logger.warning(_("ignored runtime settings", keys=unused_keys))
@@ -498,7 +502,7 @@ def _eval_task_runtime(
 def _try_task(
     cfg: config.Loader,
     logger: logging.Logger,
-    container: TaskContainer,
+    container: "runtime.task_container.TaskContainer",
     command: str,
     terminating: Callable[[], bool],
 ) -> None:
@@ -506,6 +510,8 @@ def _try_task(
     Run the task command in the container, retrying up to runtime.preemptible occurrences of
     Interrupted errors, plus up to runtime.maxRetries occurrences of any error.
     """
+    from docker.errors import BuildError as DockerBuildError  # delay heavy import
+
     max_retries = container.runtime_values.get("maxRetries", 0)
     max_interruptions = container.runtime_values.get("preemptible", 0)
     retries = 0
@@ -561,7 +567,10 @@ def _try_task(
 
 
 def _eval_task_outputs(
-    logger: logging.Logger, task: Tree.Task, env: Env.Bindings[Value.Base], container: TaskContainer
+    logger: logging.Logger,
+    task: Tree.Task,
+    env: Env.Bindings[Value.Base],
+    container: "runtime.task_container.TaskContainer",
 ) -> Env.Bindings[Value.Base]:
 
     # helper to rewrite File/Directory from in-container paths to host paths
@@ -721,7 +730,10 @@ def link_outputs(
 
 
 def _delete_work(
-    cfg: config.Loader, logger: logging.Logger, container: Optional[TaskContainer], success: bool
+    cfg: config.Loader,
+    logger: logging.Logger,
+    container: "Optional[runtime.task_container.TaskContainer]",
+    success: bool,
 ) -> None:
     opt = cfg["file_io"]["delete_work"].strip().lower()
     if container and (
@@ -737,11 +749,15 @@ def _delete_work(
 
 class _StdLib(StdLib.Base):
     logger: logging.Logger
-    container: TaskContainer
+    container: "runtime.task_container.TaskContainer"
     inputs_only: bool  # if True then only permit access to input files
 
     def __init__(
-        self, wdl_version: str, logger: logging.Logger, container: TaskContainer, inputs_only: bool
+        self,
+        wdl_version: str,
+        logger: logging.Logger,
+        container: "runtime.task_container.TaskContainer",
+        inputs_only: bool,
     ) -> None:
         super().__init__(wdl_version, write_dir=os.path.join(container.host_dir, "write_"))
         self.logger = logger
@@ -768,13 +784,23 @@ class _StdLib(StdLib.Base):
 
 class InputStdLib(_StdLib):
     # StdLib for evaluation of task inputs and command
-    def __init__(self, wdl_version: str, logger: logging.Logger, container: TaskContainer) -> None:
+    def __init__(
+        self,
+        wdl_version: str,
+        logger: logging.Logger,
+        container: "runtime.task_container.TaskContainer",
+    ) -> None:
         super().__init__(wdl_version, logger, container, True)
 
 
 class OutputStdLib(_StdLib):
     # StdLib for evaluation of task outputs
-    def __init__(self, wdl_version: str, logger: logging.Logger, container: TaskContainer) -> None:
+    def __init__(
+        self,
+        wdl_version: str,
+        logger: logging.Logger,
+        container: "runtime.task_container.TaskContainer",
+    ) -> None:
         super().__init__(wdl_version, logger, container, False)
 
         setattr(
