@@ -28,7 +28,7 @@ from .._util import (
 )
 from .._util import StructuredLogMessage as _
 from . import config, _statusbar
-from .error import OutputError, Interrupted, Terminated, CommandFailed, RunFailed, error_json
+from .error import OutputError, Interrupted, Terminated, CommandFailed
 
 
 class TaskContainer(ABC):
@@ -597,11 +597,14 @@ class SwarmContainer(TaskContainer):
             return exit_code
         finally:
             if svc:
-                try:
-                    svc.remove()
-                except:
-                    logger.exception("failed to remove docker service")
-                self.chown(logger, client, exit_code == 0)
+                for attempt in range(3):
+                    try:
+                        svc.remove()
+                        break
+                    except:
+                        logger.exception("failed to remove docker service")
+                        time.sleep(2)
+            self.chown(logger, client, exit_code == 0)
             try:
                 client.close()
             except:
@@ -618,6 +621,7 @@ class SwarmContainer(TaskContainer):
             image_attrs = client.images.get(image_tag).attrs
         except docker.errors.ImageNotFound:
             try:
+                logger.info(_("docker pull", tag=image_tag))
                 client.images.pull(image_tag)
                 image_attrs = client.images.get(image_tag).attrs
             except docker.errors.ImageNotFound:
