@@ -42,7 +42,6 @@ import threading
 from concurrent import futures
 from typing import Optional, List, Set, Tuple, NamedTuple, Dict, Union, Iterable, Callable, Any
 from contextlib import ExitStack
-import importlib_metadata
 from .. import Env, Type, Value, Tree, StdLib
 from ..Error import InputError
 from .task import run_local_task, _fspaths, link_outputs, _add_downloadable_defaults
@@ -58,7 +57,7 @@ from .._util import (
 )
 from .._util import StructuredLogMessage as _
 from . import config, _statusbar
-from .cache import CallCache
+from .cache import CallCache, new as new_call_cache
 from .error import RunFailed, Terminated, error_json
 
 
@@ -651,13 +650,9 @@ def run_local_workflow(
         write_values_json(inputs, os.path.join(run_dir, "inputs.json"), namespace=workflow.name)
 
         # query call cache
-        cache = _cache if _cache else cleanup.enter_context(CallCache(cfg, logger))
+        cache = _cache if _cache else cleanup.enter_context(new_call_cache(cfg, logger))
         cache_key = f"{workflow.name}/{workflow.digest}/{Value.digest_env(inputs)}"
-        cached = cache.get(
-            key=cache_key,
-            output_types=workflow.effective_outputs,
-            inputs=inputs,
-        )
+        cached = cache.get(cache_key, inputs, workflow.effective_outputs)
         if cached is not None:
             for outp in workflow.effective_outputs:
                 v = cached[outp.name]
@@ -680,8 +675,11 @@ def run_local_workflow(
 
         # if we're the top-level workflow, provision thread pools
         if not _thread_pools:
+            import importlib_metadata  # delayed heavy import
+
             assert not _run_id_stack
             cache.flock(logfile, exclusive=True)  # flock top-level workflow.log
+            cache.flock(run_dir, exclusive=True)
             try:
                 # log version into workflow.log
                 version = "v" + importlib_metadata.version("miniwdl")
