@@ -7,10 +7,10 @@ import shutil
 import json
 import time
 import docker
+import platform
 from testfixtures import log_capture
 from .context import WDL
 
-
 class RunnerTestCase(unittest.TestCase):
     """
     Base class for new runner test cases
@@ -160,7 +160,7 @@ class TestDirectoryIO(RunnerTestCase):
 
         assert len(outp["d_out"]) == 2
         assert os.path.islink(outp["d_out"][0])
-        assert os.path.realpath(outp["d_out"][0]) == os.path.join(self._dir, "d")
+        assert os.path.realpath(outp["d_out"][0]) == os.path.realpath(os.path.join(self._dir, "d"))
         assert os.path.isdir(outp["d_out"][1])
         assert os.path.islink(outp["d_out"][1])
         assert os.path.basename(outp["d_out"][1]) == "outdir"
@@ -173,7 +173,7 @@ class TestDirectoryIO(RunnerTestCase):
         outp = self._run(wdl, {"d": os.path.join(self._dir, "d")}, cfg=cfg)
         assert len(outp["d_out"]) == 2
         assert not os.path.islink(outp["d_out"][0])
-        assert os.path.realpath(outp["d_out"][0]) != os.path.join(self._dir, "d")
+        assert os.path.realpath(outp["d_out"][0]) != os.path.realpath(os.path.join(self._dir, "d"))
         assert os.path.isdir(outp["d_out"][1])
         assert not os.path.islink(outp["d_out"][1])
         assert os.path.basename(outp["d_out"][1]) == "outdir"
@@ -327,7 +327,7 @@ class TestDirectoryIO(RunnerTestCase):
 class TestNoneLiteral(RunnerTestCase):
     def test_none_eval(self):
         wdl = R"""
-        version development
+        version 1.1
         struct Car {
             String make
             String? model
@@ -370,7 +370,7 @@ class TestNoneLiteral(RunnerTestCase):
 class TestCallAfter(RunnerTestCase):
     def test_call_after(self):
         wdl = R"""
-        version development
+        version 1.1
         task nop {
             input {
                 Int? y = 0
@@ -405,7 +405,7 @@ class TestCallAfter(RunnerTestCase):
 
         with self.assertRaises(WDL.Error.NoSuchCall):
             self._run(R"""
-            version development
+            version 1.1
             task nop {
                 input {}
                 command {}
@@ -421,7 +421,7 @@ class TestCallAfter(RunnerTestCase):
 
         with self.assertRaises(WDL.Error.CircularDependencies):
             self._run(R"""
-            version development
+            version 1.1
             task nop {
                 input {}
                 command {}
@@ -720,6 +720,8 @@ class MiscRegressionTests(RunnerTestCase):
                 filenames.append(c)
             filenames.append(c + ''.join(random.choices(chars,k=11)))
         assert filenames == list(sorted(filenames))
+        if platform.system() == "Darwin":  # macOS is case-insensitive
+            filenames = list(set(fn.lower() for fn in filenames))
         filenames.append('ThisIs{{AVeryLongFilename }}abc...}}xzy1234567890!@{{నేనుÆды.test.ext')
 
         inputs = {"files": []}
@@ -814,7 +816,7 @@ class TestAbbreviatedCallInput(RunnerTestCase):
 
     def test_docker(self):
         caller = R"""
-        version development
+        version 1.1
         workflow caller {
             input {
                 String message
